@@ -5,7 +5,7 @@
 ;; Author: Shelagh Manton <shelagh.manton@gmail.com> with help from
 ;; Dianne Skoll
 ;; Keywords: remind configure convenience
-;; Version: 0.14
+;; Version: 0.15-dfs2
 
 ;; This program is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU General Public License
@@ -91,7 +91,6 @@
 (define-key remind-conf-mode-map "\C-c\C-x" 'rem-tomorrow)
 (define-key remind-conf-mode-map "\C-c\C-a" 'rem-days-away)
 (define-key remind-conf-mode-map "\M-j" 'remind-indent-line)
-(define-key remind-conf-mode-map (kbd "RET") 'remind-indent-line)
 (define-key remind-conf-mode-map "\C-c\C-c" 'rem-save-file)
 
 ;; syntax-table
@@ -111,12 +110,18 @@
 
 (defconst remind-keywords
   (sort
-   (list "RUN" "REM" "ONCE" "SATISFY" "BEFORE" "UNSET" "OMIT" "FIRST" "SATISFY"
-	 "OMIT" "DATE" "SKIP" "ONCE" "AFTER" "WARN" "PRIORITY" "AT" "SCHED" "IF" "ELSE" "ENDIF"
-	 "WARN" "UNTIL" "THROUGH" "SCANFROM" "DURATION" "TAG" "MSG" "MSF" "CAL" "SPECIAL" "IFTRIG"
-	 "PS" "PSFILE" "BANNER" "INCLUDE" "PUSH-OMIT-CONTEXT" "DEBUG" "DUMPVARS" "PUSH" "CLEAR" "POP"
-	 "CLEAR-OMIT-CONTEXT" "POP-OMIT-CONTEXT"  "SET" "ERRMSG" "FSET" "DUMP" "BAN" "INC" "SCAN"
-	 "EXIT" "FLUSH" "PRESERVE" "MOON" "COLOR" "UNSET")
+   (list "ADDOMIT" "AFTER" "AT" "BANNER" "BEFORE"
+         "CAL" "CLEAR-OMIT-CONTEXT" "DEBUG" "DO" "DUMPVARS"
+         "DURATION" "ELSE" "ENDIF" "ERRMSG" "EXIT" "FIRST"
+         "FLUSH" "FOURTH" "FROM" "FSET" "IF" "IFTRIG" "IN"
+         "INCLUDE" "INCLUDECMD" "LAST" "LASTDAY"
+         "LASTWORKDAY" "MAYBE-UNCOMPUTABLE" "MSF"
+         "MSG" "OMIT" "OMITFUNC" "ONCE"
+         "POP-OMIT-CONTEXT" "PRESERVE" "PRIORITY" "PS" "PSFILE"
+         "PUSH-OMIT-CONTEXT" "REM" "RUN" "SATISFY" "SCANFROM"
+         "SCHED" "SECOND" "SET" "SKIP" "SPECIAL"
+         "TAG" "THIRD" "THROUGH" "UNSET" "UNTIL"
+         "WARN")
    #'(lambda (a b) (> (length a) (length b)))))
 
 
@@ -127,12 +132,20 @@
 
 (defconst remind-builtin-variables
   (sort
-   (list "$CalcUTC" "$CalMode" "$Daemon" "$DateSep" "$DefaultPrio" "$DontFork" "$DontTrigAts" "$DontQueue"
-	 "$EndSent" "$EndSentIg" "$NumTrig" "$FirstIndent" "$FoldYear" "$FormWidth" "$HushMode"
-	 "$IgnoreOnce" "$InfDelta" "$NextMode" "$NumQueued" "$NumTrig" "$PrefixLineNo" "$PSCal" "$RunOff"
-	 "$SimpleCal" "$SortByDate" "$SortByPrio" "$MinsFromUTC" "$LatDeg" "$LatMin" "$LatSec" "$EndSent"
-	 "$EndSentIg" "$Location" "$LongDeg" "$LongMin" "$LongSec" "$MaxSatIter" "$SubsIndent" "$T" "$Td"
-	 "$Tm" "$Tw" "$Ty" "$TimeSep" "$UntimedFirst" "$U" "$Ud" "$Um" "$Uw" "$Uy")
+   (list "$Ago" "$Am" "$And" "$April" "$At" "$August" "$CalcUTC" "$CalMode" "$Daemon" "$DateSep"
+         "$DateTimeSep" "$December" "$DefaultColor" "$DefaultPrio"
+         "$DefaultTDelta" "$DeltaOffset" "$DontFork" "$DontQueue"
+         "$DontTrigAts" "$EndSent" "$EndSentIg" "$February" "$FirstIndent"
+         "$FoldYear" "$FormWidth" "$Friday" "$Fromnow" "$Hour" "$Hplu" "$HushMode" "$IgnoreOnce"
+         "$InfDelta" "$IntMax" "$IntMin" "$Is" "$January" "$July" "$June" "$LatDeg"
+         "$Latitude" "$LatMin" "$LatSec" "$Location" "$LongDeg" "$Longitude"
+         "$LongMin" "$LongSec" "$March" "$MaxSatIter" "$MaxStringLen" "$May"
+         "$MinsFromUTC" "$Minute" "$Monday" "$Mplu" "$NextMode" "$November" "$Now" "$NumQueued"
+         "$NumTrig" "$October" "$On" "$Pm" "$PrefixLineNo" "$PSCal" "$RunOff" "$Saturday"
+         "$September" "$SimpleCal" "$SortByDate" "$SortByPrio" "$SortByTime"
+         "$SubsIndent" "$Sunday" "$SysInclude" "$T" "$Td" "$Thursday" "$TimeSep" "$Tm"
+         "$Today" "$Tomorrow" "$Tuesday" "$Tw" "$Ty" "$U" "$Ud" "$Um" "$UntimedFirst" "$Uw" "$Uy"
+         "$Was" "$Wednesday")
    #'(lambda (a b) (> (length a) (length b)))))
 
 
@@ -147,15 +160,22 @@
 
 (defconst remind-builtin-functions
   (sort
-   (list "abs" "access" "args" "asc" "baseyr" "char" "choose" "coerce" "current" "date" "datetime" "datepart" 
-	 "dawn" "day" "daysinmon" "defined" "dosubst" "dusk" "easter" "easterdate" "evaltrig" "filedate" 
-	 "filedatetime" "filedir" "filename" "getenv" "hebdate" "hebday" "hebmon" "hebyear" "hour" "iif" "index" "isdst" 
-	 "isleap" "isomitted" "language" "lower" "max" "min" "minute" "minsfromutc" "mon" "monnum" "moondate" "moondatetime"
-	 "moonphase" "moontime" "msgprefix" "msgsuffix" "nonomitted" "now" "ord" "ostype" "plural" 
-	 "psmoon" "psshade" "realcurrent" "realnow" "realtoday" "sgn" "shell" "slide" "strlen" "substr" "sunrise" "sunset" "time" "timepart" 
-	 "thisyear" "today" "trigdate" "trigdatetime" "trigger" "trigger" "trigtime" "trigvalid" "typeof" "tzconvert" "upper" "value" 
-	 "version" "weekno" "wkday" "wkdaynum" "year"
-	 )
+   (list "abs" "access" "adawn" "adusk" "ampm" "args" "asc" "baseyr" "char"
+         "choose" "coerce" "current" "date" "datepart" "datetime" "dawn" "day"
+         "daysinmon" "defined" "dosubst" "dusk" "easterdate" "evaltrig"
+         "filedate" "filedatetime" "filedir" "filename" "getenv" "hebdate"
+         "hebday" "hebmon" "hebyear" "hour" "iif" "index" "isany" "isdst"
+         "isleap" "isomitted" "language" "lower" "max" "min" "minsfromutc"
+         "minute" "mon" "monnum" "moondate" "moondatetime" "moonphase"
+         "moontime" "ndawn" "ndusk" "nonomitted" "now" "ord" "ostype" "pad" "plural"
+         "psmoon" "psshade" "realcurrent" "realnow" "realtoday" "sgn" "shell"
+         "shellescape" "slide" "strlen" "substr" "sunrise" "sunset" "time"
+         "timepart" "today" "trig" "trigback" "trigdate" "trigdatetime"
+         "trigdelta" "trigduration" "trigeventduration" "trigeventstart"
+         "trigfrom" "trigger" "trigpriority" "trigrep" "trigscanfrom"
+         "trigtime" "trigtimedelta" "trigtimerep" "triguntil" "trigvalid"
+         "typeof" "tzconvert" "upper" "value" "version" "weekno" "wkday"
+         "wkdaynum" "year")
    #'(lambda (a b) (> (length a) (length b)))))
 
 ;;; faces
@@ -244,12 +264,35 @@
 :group 'remind-conf
 )
 
+(defconst remind-keywords-regex (regexp-opt remind-keywords 'words))
+(defconst remind-type-keywords-regex (regexp-opt remind-type-keywords 'words))
+(defconst remind-builtin-variables-regex (regexp-opt remind-builtin-variables 'words))
+(defconst remind-builtin-functions-regex (regexp-opt remind-builtin-functions 'words))
+(defconst remind-time-words-regex (regexp-opt remind-time-words 'words))
+
+;; Case-insensitive matching functions
+(defun remind-keywords-matcher (limit)
+  (let ((case-fold-search t))
+    (re-search-forward remind-keywords-regex limit 'no-error)))
+(defun remind-type-keywords-matcher (limit)
+  (let ((case-fold-search t))
+    (re-search-forward remind-type-keywords-regex limit 'no-error)))
+(defun remind-builtin-variables-matcher (limit)
+  (let ((case-fold-search t))
+    (re-search-forward remind-builtin-variables-regex limit 'no-error)))
+(defun remind-builtin-functions-matcher (limit)
+  (let ((case-fold-search t))
+    (re-search-forward remind-builtin-functions-regex limit 'no-error)))
+(defun remind-time-words-matcher (limit)
+  (let ((case-fold-search t))
+    (re-search-forward remind-time-words-regex limit 'no-error)))
+
 ;; keywords
 
 (defconst remind-conf-font-lock-keywords-1
   (list
    '("^[\;\#]\\s-+.*$" . remind-comment-face)
-   (cons (regexp-opt remind-keywords 'words) remind-conf-keyword-face)
+   '(remind-keywords-matcher . remind-conf-keyword-face)
    '("%[\"_]" . font-lock-warning-face)
    '("\\(%[a-mops-w]\\)" . remind-conf-substitutes-face)
    '("\"[^\"]*\"" . remind-string-face))
@@ -258,15 +301,15 @@
 (defconst remind-conf-font-lock-keywords-2
   (append remind-conf-font-lock-keywords-1
 	  (list
-	   (cons (regexp-opt remind-time-words 'words) remind-time-face)
-	   (cons (regexp-opt remind-builtin-functions 'words) remind-conf-command-face)
+           '(remind-time-words-matcher . remind-time-face)
+           '(remind-builtin-functions-matcher . remind-conf-command-face)
 	   '("%$" . remind-conf-endline-face)))
   "Additional commands to highlight in `remind-conf-mode'.")
 
 (defconst remind-conf-font-lock-keywords-3
   (append remind-conf-font-lock-keywords-2
 	  (list
-	   (cons (regexp-opt remind-type-keywords 'words) remind-conf-type-face)
+           '(remind-type-keywords-matcher . remind-conf-type-face)
 	   '("\[[a-zA-Z]\\{3,6\\}\]" . remind-conf-color-face)
 	   '("\\s-+\\([12][0-9]\\|3[01]\\|0?[0-9]\\)\\s-+" . remind-conf-substitutes-face);better date regexp
 	   '("\\s-+\\([12][09][0-9][0-9][-/]\\(0[1-9]\\|1[0-2]\\)[-/]\\([12][0-9]\\|0[1-9]\\|3[01]\\)\\)\\s-+" . remind-time-face) ;; pseudo ISO 8601 date format.
@@ -274,7 +317,7 @@
 	   '("\\s-+\\(\\(?:20\\|19\\)[0-9][0-9]\\)\\s-+" . remind-conf-substitutes-face);years
 	   '("\\s-+\\(2[0-4]\\|[01]?[0-9][.:][0-5][0-9]\\)\\s-+" . remind-conf-substitutes-face);24hour clock, more precise
 	   '("\\s-+\\([+-][+-]?[1-9][0-9]*\\)\\s-+" 1 remind-conf-delta-face prepend)
-	   (cons (regexp-opt remind-builtin-variables 'words) remind-conf-variable-face)))
+           '(remind-builtin-variables-matcher . remind-conf-variable-face)))
   "The ultimate in highlighting experiences for `remind-conf-mode'.")
 
 ;;YYYY-MM-DD@hh:mm, YYYY-MM-DD@hh.mm, YYYY/MM/DD@hh:mm and YYYY/MM/DD@hh.mm
@@ -462,12 +505,11 @@ Acts on the region or places point where it needs to be."
 ;; finally the derived mode.
 
 ;;;###autoload
-(define-derived-mode remind-conf-mode text-mode "REM"
+(define-derived-mode remind-conf-mode text-mode "Remind Conf Mode"
   "Major mode for editing remind calendar configuration files.
 
 \\{remind-conf-mode-map}"
   :syntax-table remind-conf-syntax-table
-  (set (make-local-variable 'font-lock-keywords-case-fold-search) t) ;this is not working atm 2009-04-13
   (set (make-local-variable 'font-lock-defaults) '(remind-conf-font-lock-keywords))
   (set (make-local-variable 'comment-start) ";")
   (set (make-local-variable 'comment-start) "#")
