@@ -70,7 +70,7 @@
 #define E_MON_TWICE	     51
 #define E_DAY_TWICE	     52
 #define E_UNKNOWN_TOKEN	     53
-#define E_SPEC_MON_DAY	     54
+#define E_SPEC_MON    	     54
 #define E_2MANY_PART	     55
 #define E_2MANY_FULL	     56
 #define E_PUSH_NOPOP	     57
@@ -187,7 +187,7 @@ EXTERN char *ErrMsg[]
     "Month specified twice",
     "Day specified twice",
     "Unknown token",
-    "Must specify month and day in OMIT command",
+    "Must specify month in OMIT command",
     "Too many partial OMITs",
     "Too many full OMITs",
     "Warning: PUSH-OMIT-CONTEXT without matching POP-OMIT-CONTEXT",
