@@ -5,7 +5,7 @@
 /*  This file contains routines to parse and evaluate          */
 /*  expressions.                                               */
 /*                                                             */
-/*  Copyright 1992-2021 by Dianne Skoll                        */
+/*  Copyright 1992-2022 by Dianne Skoll                        */
 /*                                                             */
 /***************************************************************/
 
@@ -44,21 +44,32 @@ static int MakeValue (char const *s, Value *v, Var *locals, ParsePtr p);
 
 /* Binary operators - all left-associative */
 
+/* Operator precedence:
+ *   Highest:    Unary -    Unary !
+ *               /   %
+ *               +   -
+ *               <   <=   >   >=
+ *               ==   !=
+ *               &&
+ *    Lowest:    ||
+ *
+ */
+
 /* Make SURE they are sorted lexically... this may die on an EBCDIC
    system... */
 
 Operator BinOp[] = {
     { "!=", 15, BIN_OP, NotEqual },
-    { "%", 20, BIN_OP, Mod },
+    { "%",  20, BIN_OP, Mod },
     { "&&", 14, BIN_OP, LogAND },
-    { "*", 20, BIN_OP, Multiply },
-    { "+", 18, BIN_OP, Add },
-    { "-", 18, BIN_OP, Subtract },
-    { "/", 20, BIN_OP, Divide },
-    { "<", 16, BIN_OP, LessThan },
+    { "*",  20, BIN_OP, Multiply },
+    { "+",  18, BIN_OP, Add },
+    { "-",  18, BIN_OP, Subtract },
+    { "/",  20, BIN_OP, Divide },
+    { "<",  16, BIN_OP, LessThan },
     { "<=", 16, BIN_OP, LessOrEqual },
     { "==", 15, BIN_OP, EqualTo },
-    { ">", 16, BIN_OP, GreaterThan },
+    { ">",  16, BIN_OP, GreaterThan },
     { ">=", 16, BIN_OP, GreaterOrEqual },
     { "||", 12, BIN_OP, LogOR },
 };
@@ -66,16 +77,19 @@ Operator BinOp[] = {
 
 /* These ones must be sorted too. */
 Operator UnOp[] = {
-    { "!", 22, UN_OP, LogNot },
-    { "-", 22, UN_OP, UnMinus },
+    { "!",  22, UN_OP, LogNot },
+    { "-",  22, UN_OP, UnMinus },
 };
 #define NUM_UN_OPS (sizeof(UnOp) / sizeof(Operator))
 
 extern BuiltinFunc Func[];
 
-Operator OpStack[OP_STACK_SIZE];
-Value    ValStack[VAL_STACK_SIZE];
-int OpStackPtr, ValStackPtr;
+static Operator OpStack[OP_STACK_SIZE];
+static int      OpStackPtr = 0;
+
+/* ValStack can't be static - needed by funcs.c */
+       Value    ValStack[VAL_STACK_SIZE];
+       int      ValStackPtr = 0;
 
 /***************************************************************/
 /*                                                             */
@@ -115,12 +129,13 @@ static int DebugPerform(Operator *op)
 /*  Clean the stack after an error occurs.                     */
 /*                                                             */
 /***************************************************************/
-static void CleanStack(void)
+static void CleanStack(int old_op_stack_ptr, int old_val_stack_ptr)
 {
     int i;
 
-    for (i=0; i<ValStackPtr; i++) DestroyValue(ValStack[i]);
-    ValStackPtr = 0;
+    for (i=old_val_stack_ptr; i<ValStackPtr; i++) DestroyValue(ValStack[i]);
+    ValStackPtr = old_val_stack_ptr;
+    OpStackPtr = old_op_stack_ptr;
 }
 
 /***************************************************************/
@@ -306,8 +321,8 @@ int EvalExpr(char const **e, Value *v, ParsePtr p)
 {
     int r;
 
-    OpStackPtr = 0;
-    ValStackPtr = 0;
+    int old_op_stack_ptr = OpStackPtr;
+    int old_val_stack_ptr = ValStackPtr;
 
     r = Evaluate(e, NULL, p);
 
@@ -316,11 +331,15 @@ int EvalExpr(char const **e, Value *v, ParsePtr p)
     DBufFree(&ExprBuf);
 
     if (r) {
-	CleanStack();
+	CleanStack(old_op_stack_ptr, old_val_stack_ptr);
+        /* fprintf(stderr, "O=%d V=%d\n", OpStackPtr, ValStackPtr); */
 	return r;
     }
-    *v = *ValStack;
-    ValStack[0].type = ERR_TYPE;
+    OpStackPtr = old_op_stack_ptr;
+    ValStackPtr = old_val_stack_ptr;
+    *v = ValStack[old_val_stack_ptr];
+    ValStack[old_val_stack_ptr].type = ERR_TYPE;
+    /* fprintf(stderr, "O=%d V=%d\n", OpStackPtr, ValStackPtr); */
     return r;
 }
 
@@ -766,12 +785,11 @@ static int Add(void)
 
 /* If both are ints, just add 'em */
     if (v2.type == INT_TYPE && v1.type == INT_TYPE) {
-        int old = v1.v.val;
-	v1.v.val += v2.v.val;
         /* Check for overflow */
-        if (_private_add_overflow(v1.v.val, v2.v.val, old)) {
+        if (_private_add_overflow(v1.v.val, v2.v.val)) {
             return E_2HIGH;
         }
+	v1.v.val += v2.v.val;
 	PushValStack(v1);
 	return OK;
     }
@@ -779,9 +797,8 @@ static int Add(void)
 /* If it's a date plus an int, add 'em */
     if ((v1.type == DATE_TYPE && v2.type == INT_TYPE) ||
 	(v1.type == INT_TYPE && v2.type == DATE_TYPE)) {
-        int old = v1.v.val;
+        if (_private_add_overflow(v1.v.val, v2.v.val)) return E_DATE_OVER;
 	v1.v.val += v2.v.val;
-        if (_private_add_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
 	if (v1.v.val < 0) return E_DATE_OVER;
 	v1.type = DATE_TYPE;
 	PushValStack(v1);
@@ -791,9 +808,8 @@ static int Add(void)
 /* If it's a datetime plus an int or a time, add 'em */
     if ((v1.type == DATETIME_TYPE && (v2.type == INT_TYPE || v2.type == TIME_TYPE)) ||
 	((v1.type == INT_TYPE || v1.type == TIME_TYPE) && v2.type == DATETIME_TYPE)) {
-        int old = v1.v.val;
+        if (_private_add_overflow(v1.v.val, v2.v.val)) return E_DATE_OVER;
 	v1.v.val += v2.v.val;
-        if (_private_add_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
 	if (v1.v.val < 0) return E_DATE_OVER;
 	v1.type = DATETIME_TYPE;
 	PushValStack(v1);
@@ -805,9 +821,8 @@ static int Add(void)
     if ((v1.type == TIME_TYPE && v2.type == INT_TYPE) ||
 	(v1.type == INT_TYPE && v2.type == TIME_TYPE) ||
 	(v1.type == TIME_TYPE && v2.type == TIME_TYPE)) {
-        int old = v1.v.val;
+        if (_private_add_overflow(v1.v.val, v2.v.val)) return E_DATE_OVER;
 	v1.v.val += v2.v.val;
-        if (_private_add_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
 	v1.v.val = v1.v.val % MINUTES_PER_DAY;
 	if (v1.v.val < 0) v1.v.val += MINUTES_PER_DAY;
 	v1.type = TIME_TYPE;
@@ -868,18 +883,16 @@ static int Subtract(void)
 
     /* If they're both INTs, do subtraction */
     if (v1.type == INT_TYPE && v2.type == INT_TYPE) {
-        int old = v1.v.val;
+        if (_private_sub_overflow(v1.v.val, v2.v.val)) return E_2HIGH;
 	v1.v.val -= v2.v.val;
-        if (_private_sub_overflow(v1.v.val, v2.v.val, old)) return E_2HIGH;
 	PushValStack(v1);
 	return OK;
     }
 
     /* If it's a date minus an int, do subtraction, checking for underflow */
     if (v1.type == DATE_TYPE && v2.type == INT_TYPE) {
-        int old = v1.v.val;
+        if (_private_sub_overflow(v1.v.val, v2.v.val)) return E_DATE_OVER;
 	v1.v.val -= v2.v.val;
-        if (_private_sub_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
 	if (v1.v.val < 0) return E_DATE_OVER;
 	PushValStack(v1);
 	return OK;
@@ -888,9 +901,8 @@ static int Subtract(void)
     /* If it's a datetime minus an int or a time, do subtraction,
      * checking for underflow */
     if (v1.type == DATETIME_TYPE && (v2.type == INT_TYPE || v2.type == TIME_TYPE)) {
-        int old = v1.v.val;
+        if (_private_sub_overflow(v1.v.val, v2.v.val)) return E_DATE_OVER;
 	v1.v.val -= v2.v.val;
-        if (_private_sub_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
 	if (v1.v.val < 0) return E_DATE_OVER;
 	PushValStack(v1);
 	return OK;
@@ -908,9 +920,8 @@ static int Subtract(void)
     if ((v1.type == TIME_TYPE && v2.type == TIME_TYPE) ||
 	(v1.type == DATETIME_TYPE && v2.type == DATETIME_TYPE) ||
 	(v1.type == DATE_TYPE && v2.type == DATE_TYPE)) {
-        int old = v1.v.val;
+        if (_private_sub_overflow(v1.v.val, v2.v.val)) return E_DATE_OVER;
 	v1.v.val -= v2.v.val;
-        if (_private_sub_overflow(v1.v.val, v2.v.val, old)) return E_DATE_OVER;
 	v1.type = INT_TYPE;
 	PushValStack(v1);
 	return OK;
@@ -945,11 +956,8 @@ static int Multiply(void)
             (v1.v.val == -1 && v2.v.val == INT_MIN)) {
             return E_2HIGH;
         }
-        int old = v1.v.val;
+        if (_private_mul_overflow(v1.v.val, v2.v.val)) return E_2HIGH;
 	v1.v.val *= v2.v.val;
-        if (v2.v.val != 0) {
-            if (_private_div(v1.v.val, v2.v.val) != old) return E_2HIGH;
-        }
 	PushValStack(v1);
 	return OK;
     }
@@ -1108,13 +1116,16 @@ static int LogOR(void)
 	return r;
     }
 
-    if (v1.type == INT_TYPE && v2.type == INT_TYPE) {
-	v1.v.val = (v1.v.val || v2.v.val) ? 1 : 0;
-	PushValStack(v1);
-	return OK;
+    if (v1.type != v2.type || v1.type == STR_TYPE || v2.type == STR_TYPE) {
+        DestroyValue(v1); DestroyValue(v2);
+        return E_BAD_TYPE;
     }
-    DestroyValue(v1); DestroyValue(v2);
-    return E_BAD_TYPE;
+
+    if (v1.v.val == 0) {
+        v1.v.val = v2.v.val;
+    }
+    PushValStack(v1);
+    return OK;
 }
 
 /***************************************************************/
@@ -1135,13 +1146,15 @@ static int LogAND(void)
 	return r;
     }
 
-    if (v1.type == INT_TYPE && v2.type == INT_TYPE) {
-	v1.v.val = (v1.v.val && v2.v.val) ? 1 : 0;
-	PushValStack(v1);
-	return OK;
+    if (v1.type != v2.type || v1.type == STR_TYPE || v2.type == STR_TYPE) {
+        DestroyValue(v1); DestroyValue(v2);
+        return E_BAD_TYPE;
     }
-    DestroyValue(v1); DestroyValue(v2);
-    return E_BAD_TYPE;
+    if (v1.v.val != 0) {
+        v1.v.val = v2.v.val;
+    }
+    PushValStack(v1);
+    return OK;
 }
 
 /***************************************************************/
@@ -1155,9 +1168,8 @@ static int UnMinus(void)
 {
     Value *v = &ValStack[ValStackPtr-1];
     if (v->type != INT_TYPE) return E_BAD_TYPE;
-    int old = v->v.val;
+    if (v->v.val == INT_MIN) return E_2HIGH;
     v->v.val = -v->v.val;
-    if (_private_unminus_overflow(old, v->v.val)) return E_2HIGH;
     return OK;
 }
 
