@@ -5,8 +5,8 @@
 /*  Support for the Icelandic language.                        */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
-/*  Translated by Bjrn Davsson (bjossi@snerpa.is)           */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Translated by Björn Davíðsson (bjossi@snerpa.is)           */
 /*                                                             */
 /***************************************************************/
 
@@ -15,59 +15,56 @@
 
 /* Day names */
 #define L_SUNDAY "sunnudagur"
-#define L_MONDAY "mnudagur"
-#define L_TUESDAY "rijudagur"
-#define L_WEDNESDAY "mivikudagur"
+#define L_MONDAY "mánudagur"
+#define L_TUESDAY "þriðjudagur"
+#define L_WEDNESDAY "miðvikudagur"
 #define L_THURSDAY "fimmtudagur"
-#define L_FRIDAY "fstudagur"
+#define L_FRIDAY "föstudagur"
 #define L_SATURDAY "laugardagur"
 
 /* Month names */
-#define L_JAN "janar"
-#define L_FEB "febrar"
+#define L_JAN "janúar"
+#define L_FEB "febrúar"
 #define L_MAR "mars"
-#define L_APR "aprl"
-#define L_MAY "ma"
-#define L_JUN "jn"
-#define L_JUL "jl"
-#define L_AUG "gst"
+#define L_APR "apríl"
+#define L_MAY "maí"
+#define L_JUN "júní"
+#define L_JUL "júlí"
+#define L_AUG "ágúst"
 #define L_SEP "september"
-#define L_OCT "oktber"
-#define L_NOV "nvember"
+#define L_OCT "október"
+#define L_NOV "nóvember"
 #define L_DEC "desember"
 
 /* Today and tomorrow */
-#define L_TODAY " dag"
-#define L_TOMORROW " morgun"
+#define L_TODAY "í dag"
+#define L_TOMORROW "á morgun"
 
 /* The default banner */
-#define L_BANNER "Minnisatrii: %w, %d%s %m, %y%o:"
+#define L_BANNER "Minnisatriði: %w, %d%s %m, %y%o:"
 
 /* "am" and "pm" */
 #define L_AM "fh"
 #define L_PM "eh"
 
-/*** The following are only used in dosubst.c ***/
-#ifdef L_IN_DOSUBST
-
 /* Ago and from now */
-#define L_AGO "san"
-#define L_FROMNOW "fr v n"
+#define L_AGO "síðan"
+#define L_FROMNOW "frá því nú"
 
 /* "in %d days' time" */
 #define L_INXDAYS "eftir %d daga"
 
 /* "on" as in "on date..." */
-#define L_ON "ann"
+#define L_ON "þann"
 
 /* Pluralizing - this is a problem for many languages and may require
    a more drastic fix */
 #define L_PLURAL "a"
 
 /* Minutes, hours, at, etc */
-#define L_NOW "nna"
+#define L_NOW "núna"
 #define L_AT "kl."
-#define L_MINUTE "mntu"
+#define L_MINUTE "mínútu"
 #define L_HOUR "klukkustund"
 #define L_IS "er"
 #define L_WAS "var"
@@ -79,5 +76,3 @@
 
 /* Define any overrides here, such as L_ORDINAL_OVERRIDE, L_A_OVER, etc.
    See the file dosubst.c for more info. */
-
-#endif /* L_IN_DOSUBST */
