@@ -8,7 +8,7 @@
 /*                                                             */
 /*  This file is part of REMIND.                               */
 /*                                                             */
-/*  REMIND is Copyright (C) 1992-2021 by Dianne Skoll          */
+/*  REMIND is Copyright (C) 1992-2022 by Dianne Skoll          */
 /*  This file is Copyright (C) 1996-1998 by Liviu Daia         */
 /*                                                             */
 /***************************************************************/
@@ -50,9 +50,6 @@
 #define L_AM "am"
 #define L_PM "pm"
 
-#ifdef L_IN_DOSUBST
-/*** The following are only used in dosubst.c ***/
-
 /* Ago and from now */
 #define L_AGO "acum"
 #define L_FROMNOW "peste"
@@ -76,6 +73,7 @@
 #define L_WAS "a fost"
 /* What to add to make "minute" plural */
 #define L_MPLU "e"
+#define L_HPLU "e"
 
 /* What to add to make "hour" plural */
 #define L_HPLU_OVER hplu = (hdiff == 1 ? "ă" : "e");
@@ -104,5 +102,3 @@
             else							\
                 sprintf(s, "%s %d %s%s %s %d %s%s", when, hdiff, 	\
 			L_HOUR, hplu, L_AND, mdiff, L_MINUTE, mplu);
-
-#endif /* L_IN_DOSUBST */
