@@ -170,10 +170,11 @@ static int latitude_func(int do_set, Value *val)
 static int trig_date_func(int do_set, Value *val)
 {
     UNUSED(do_set);
-    val->type = DATE_TYPE;
     if (!LastTrigValid) {
+        val->type = INT_TYPE;
 	val->v.val = 0;
     } else {
+        val->type = DATE_TYPE;
 	val->v.val = LastTriggerDate;
     }
     return OK;
@@ -849,6 +850,7 @@ static SysVar SysVarArr[] = {
     {"SortByTime",     0,  INT_TYPE,     &SortByTime,          0,      0,    NULL },
     {"SubsIndent",     1,  INT_TYPE,     &SubsIndent,          0,      132,  NULL },
     {"Sunday",         1,  STR_TYPE,     &DynamicDayName[6],   0,      0,    NULL },
+    {"SuppressLRM",    1,  INT_TYPE,     &SuppressLRM,         0,      1,    NULL },
     {"SysInclude",     0,  STR_TYPE,     &SysDir,              0,      0,    NULL },
     {"T",              0,  SPECIAL_TYPE, trig_date_func,       0,      0,    NULL },
     {"Td",             0,  SPECIAL_TYPE, trig_day_func,        0,      0,    NULL },
